@@ -1,13 +1,18 @@
-from django import VERSION
+from typing import Optional
+
 from django import template
-from django.template.base import Parser, Token, TOKEN_BLOCK
+from django.template.base import Parser, Token
 from django.forms import ChoiceField
 from django.utils.safestring import mark_safe
 
+from .compat import TOKEN_BLOCK
 from .templatetags.sitetree import sitetree_tree
 from .utils import get_tree_model, get_tree_item_model
 from .settings import ITEMS_FIELD_ROOT_ID
 
+if False:  # pragma: nocover
+    from .models import TreeItemBase, TreeBase  # noqa
+
 
 MODEL_TREE_CLASS = get_tree_model()
 MODEL_TREE_ITEM_CLASS = get_tree_item_model()
@@ -21,15 +26,35 @@ class TreeItemChoiceField(ChoiceField):
     Use `initial` kwarg to set initial sitetree item by its ID.
 
     """
+    template: str = 'admin/sitetree/tree/tree_combo.html'
+    root_title: str = '---------'
 
-    template = 'admin/sitetree/tree/tree_combo.html'
-    root_title = '---------'
-
-    def __init__(self, tree, required=True, widget=None, label=None, initial=None, help_text=None, *args, **kwargs):
-        super(TreeItemChoiceField, self).__init__(
+    def __init__(
+            self,
+            tree: 'TreeBase' = None,
+            required: bool = True,
+            widget=None,
+            label=None,
+            initial=None,
+            help_text=None,
+            *args, **kwargs
+    ):
+        super().__init__(
             required=required, widget=widget, label=label, initial=initial,
             help_text=help_text, *args, **kwargs)
 
+        self.tree = None
+        self.choices_init(tree)
+
+    def choices_init(self, tree: Optional['TreeBase']):
+        """Initialize choices for the given tree.
+
+        :param tree:
+
+        """
+        if not tree:
+            return
+
         if isinstance(tree, MODEL_TREE_CLASS):
             tree = tree.alias
 
@@ -38,10 +63,10 @@ class TreeItemChoiceField(ChoiceField):
 
     def _build_choices(self):
         """Build choices list runtime using 'sitetree_tree' tag"""
-        tree_token = u'sitetree_tree from "%s" template "%s"' % (self.tree, self.template)
+        tree_token = f'sitetree_tree from "{self.tree}" template "{self.template}"'
 
         context_kwargs = {'current_app': 'admin'}
-        context = template.Context(context_kwargs) if VERSION >= (1, 8) else template.Context(**context_kwargs)
+        context = template.Context(context_kwargs)
         context.update({'request': object()})
 
         choices_str = sitetree_tree(
